#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <stdio.h>
#include <setjmp.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <syslog.h>
#include <strings.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include "xmlparse.h"

/*
**  Arrange to use either varargs or stdargs
*/

#define MAXSHORTSTR	203		/* max short string length */
#define QUAD_T	unsigned long long

#ifdef __STDC__

#include <stdarg.h>

# define VA_LOCAL_DECL	va_list ap;
# define VA_START(f)	va_start(ap, f)
# define VA_END		va_end(ap)

#else /* __STDC__ */

# include <varargs.h>

# define VA_LOCAL_DECL	va_list ap;
# define VA_START(f)	va_start(ap)
# define VA_END		va_end(ap)

#endif /* __STDC__ */

#ifndef INCL_COMMON_H
#define INCL_COMMON_H

typedef struct pool_struct
{
    int size;
    void *block;
    struct pool_struct *next;
    int key; /* pool-unique external key to find block back */
} _pool, *pool;

pool pool_new(); /* creates a new memory pool */
void *pmalloc(pool p, int size); /* wrapper around malloc, stores pointer to new memory for later reference in the pool */
char *pstrdup(pool p, const char *src); /* wrapper around strdup, gains mem from pool */
void pool_free(pool p); /* frees all the data on the pool and deletes the pool itself */
void *pmalloc_key(pool p, int key, int size); /* key is a unique identifier, searches pool for matching key and returns it or creates new block and sets key, if size=0 it just checks for the key and returns it or NULL */


#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 64
#endif

#define NETSOCKET_SERVER 0
#define NETSOCKET_CLIENT 1
#define NETSOCKET_UDP 2

#ifndef WIN32
int make_netsocket(u_short port, char *host, int type);
struct in_addr *make_addr(char *host);
#endif


char *j_strdup(const char *str); /* provides NULL safe strdup wrapper */
int j_strcmp(const char *a, const char *b); /* provides NULL safe strcmp wrapper */

/* string pools */
struct spool_node
{
    char *c;
    struct spool_node *next;
};

typedef struct spool_struct
{
    pool p;
    int len;
    struct spool_node *last;
    struct spool_node *first;
} *spool;

spool spool_new(pool p); /* create a string pool */
void spooler(spool s, ...); /* append all the char * args to the pool, terminate args with s again */
char *spool_print(spool s); /* return a big string */
void spool_add(spool s, char *str); /* add a single char to the pool */

#define NTYPE_TAG    0
#define NTYPE_ATTRIB 1
#define NTYPE_CDATA  2

#define NTYPE_LAST   2
#define NTYPE_UNDEF  -1

/* -------------------------------------------------------------------------- 
   Node structure. Do not use directly! Always use accessor macros 
   and methods!
   -------------------------------------------------------------------------- */
typedef struct xmlnode_t
{
     char*               name;
     unsigned short      type;
     char*               data;
     int                 data_sz;
     int                 complete;
     pool               p;
     struct xmlnode_t*  parent;
     struct xmlnode_t*  firstchild; 
     struct xmlnode_t*  lastchild;
     struct xmlnode_t*  prev; 
     struct xmlnode_t*  next;
     struct xmlnode_t*  firstattrib;
     struct xmlnode_t*  lastattrib;
} _xmlnode, *xmlnode;

/* Node creation routines */
xmlnode  xmlnode_new_tag(const char* name);
xmlnode  xmlnode_insert_tag(xmlnode parent, const char* name); 
xmlnode  xmlnode_insert_cdata(xmlnode parent, const char* CDATA, unsigned int size);
void     xmlnode_insert_node(xmlnode parent, xmlnode node); 
xmlnode  xmlnode_str(char *str, int len);
xmlnode  xmlnode_file(char *file);
xmlnode  xmlnode_dup(xmlnode x); /* duplicate x */

/* Node Memory Pool */
pool xmlnode_pool(xmlnode node);
xmlnode _xmlnode_new(pool p, const char *name, unsigned int type);

/* Node editing */
void xmlnode_hide(xmlnode child);
void xmlnode_hide_attrib(xmlnode parent, const char *name);

/* Node deletion routine, also frees the node pool! */
void xmlnode_free(xmlnode node);

/* Locates a child tag by name and returns it */
xmlnode  xmlnode_get_tag(xmlnode parent, const char* name);

/* Attribute accessors */
void     xmlnode_put_attrib(xmlnode owner, const char* name, const char* value);
char*    xmlnode_get_attrib(xmlnode owner, const char* name);
void     xmlnode_put_expat_attribs(xmlnode owner, const char** atts);

/* Node traversal routines */
xmlnode  xmlnode_get_firstattrib(xmlnode parent);
xmlnode  xmlnode_get_firstchild(xmlnode parent);
xmlnode  xmlnode_get_lastchild(xmlnode parent);
xmlnode  xmlnode_get_nextsibling(xmlnode sibling);
xmlnode  xmlnode_get_prevsibling(xmlnode sibling);
xmlnode  xmlnode_get_parent(xmlnode node);

/* Node information routines */
char*    xmlnode_get_name(xmlnode node);
char*    xmlnode_get_data(xmlnode node);
int      xmlnode_get_datasz(xmlnode node);
int      xmlnode_get_type(xmlnode node);

int      xmlnode_has_children(xmlnode node);
int      xmlnode_has_attribs(xmlnode node);

char*    xmlnode2str(xmlnode node); /* returns memory from the node pool */
void     _xmlnode2str(spool s, xmlnode node);
void     _xmlnode2str_tag(spool s, xmlnode node);
int      xmlnode_cmp(xmlnode a, xmlnode b); /* compares a and b for equality */

int      xmlnode2file(char *file, xmlnode node); /* writes node to file */

/* Expat callbacks */
void expat_startElement(void* userdata, const char* name, const char** atts);
void expat_endElement(void* userdata, const char* name);
void expat_charData(void* userdata, const char* s, int len);


typedef struct jid_struct
{ /* type=user/resource@server */
    pool p;
    char *resource;
    char *user;
    char *server;
    char *type;
    char *full;
    struct jid_struct *next; /* for lists of jids */
} *jid;

#define JID_RESOURCE 0
#define JID_USER 1
#define JID_SERVER 2
#define JID_TYPE 3
#define JID_FULL 4

jid jid_new(pool p, char *idstr); /* Creates a jabber id from the idstr */
void jid_set(jid id, char *str, int item); /* allows you to individually set jid components */
char *jid_full(jid id); /* builds a string type=user/resource@server from the jid data */
int jid_cmp(jid a, jid b); /* compares two jid's, returns 0 for perfect match */
jid jid_append(jid a, jid b); /* appending b to a (list), no dups */

#define JPACKET_UNKNOWN 0
#define JPACKET_MESSAGE 1
#define JPACKET_PRESENCE 2
#define JPACKET_IQ 3

typedef struct jpacket_struct
{
    int type;
    int flag;
    void *aux1;
    xmlnode x;
    jid to;
    jid from;
    pool p;
} *jpacket, _jpacket;

jpacket jpacket_new(xmlnode x); /* Creates a jabber packet from the xmlnode */


#endif /* INCL_COMMON_H */
