/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "common.h"

jid jid_new(pool p, char *idstr)
{
    char *server, *resource, *type, *str;
    jid id;

    if(p == NULL || idstr == NULL)
        return NULL;

    /* type:user@server/resource */

    str = pstrdup(p, idstr);

    id = pmalloc(p,sizeof(struct jid_struct));
    id->full = id->server = id->user = id->resource = id->type = NULL;
    id->p = p;
    id->next = NULL;

    resource = strstr(str,"/");
    if(resource != NULL)
    {
        *resource = '\0';
        ++resource;
        id->resource = resource;
    }
    
    type = strstr(str,":");
    if(type != NULL)
    {
        *type = '\0';
        ++type;
        id->type = str;
        str = type;
    }

    server = strstr(str,"@");
    if(server == NULL)
    { /* if there's no @, it's just the server address */
        id->server = str;
        return id;
    }else{
        *server = '\0';
        ++server;
        id->server = server;
        id->user = str;
    }

    return id;
}

void jid_set(jid id, char *str, int item)
{
    if(id == NULL)
        return;

    /* invalidate the cached copy */
    id->full = NULL;

    switch(item)
    {
        case JID_RESOURCE:
            id->resource = pstrdup(id->p, str);
            break;
        case JID_USER:
            id->user = pstrdup(id->p, str);
            break;
        case JID_SERVER:
            id->server = pstrdup(id->p, str);
            break;
        case JID_TYPE:
            id->type = pstrdup(id->p, str);
            break;
    }
}

char *jid_full(jid id)
{
    spool s;

    if(id == NULL)
        return NULL;

    /* use cached copy */
    if(id->full != NULL)
        return id->full;

    s = spool_new(id->p);
 
    if(id->type != NULL)
        spooler(s, id->type, ":",s);
    else
        spool_add(s, "jabber:");

    if(id->user != NULL)
        spooler(s, id->user,"@",s);

    spool_add(s, id->server);

    if(id->resource != NULL)
        spooler(s, "/",id->resource,s);

    id->full = spool_print(s);
    return id->full;
}

int jid_cmp(jid a, jid b)
{
    int ret = 0;

    if(a == NULL || b == NULL)
        return -1;

    if(a->resource != NULL)
        ret += j_strcmp(a->resource, b->resource);
    if(a->user != NULL)
        ret += j_strcmp(a->user, b->user);
    if(a->server != NULL)
        ret += j_strcmp(a->server, b->server);
    if(a->type != NULL)
        ret += j_strcmp(a->type, b->type);

    return ret;
}

jid jid_append(jid a, jid b)
{
    jid next;

    if(a == NULL)
        return b;

    next = a;
    while(next != NULL)
    {
        /* check for dups */
        if(jid_cmp(next,b) == 0)
            return a;
        if(next->next == NULL)
            next->next = b;
        next = next->next;
    }
    return a; /* never reached */
}
