/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#ifndef INCL_TSTREAM_H
#define INCL_TSTREAM_H

typedef struct tstream_buffer_struct
{
    pth_message_t head; /* the standard pth message header */
    char *data;
    int data_sz;
    int sent;
    void *obj; /* can point to the obj that created the data being sent, in the same pool */
    pool p;
} *tstream_buffer;

typedef void (*tstream_handler_read)(thread t, void *buffer, int bufsz);
typedef void (*tstream_handler_err)(thread t, tstream_buffer buffer);

typedef struct tstream_struct
{
    int fd, port;
    char *host;

    tstream_handler_read h_read; /* Function pointer for handling read data */
    tstream_handler_err h_err; /* Handles data left in the write buffer when the thread gets killed */
    void *data, *data1, *data2, *data3, *data4, *data5; /* Used by the handlers/internally */

    void *data_aux1, *data_aux2, *data_aux3, *data_aux4, *data_aux5; /* For general purpose usage within thread */

    tstream_buffer write; /* The current data being written */
    pth_msgport_t write_mp; /* Pth message port for unwritten blocked data */

} _tstream, *tstream;

/* starts tstream on open descriptor */
thread tstream_new(int fd, char *host, tstream_handler_read read, tstream_handler_err err, void *data);

/* uses adns if available to resolve the host and connect to the port
 * attempts to connect to port on A first, failing that, does a reverse-MX sort lookup attempting to connect to each entry
 */
thread tstream_connect(int port, char *host, tstream_handler_read read, tstream_handler_err err, void *data);

/* Specifies new data to be written to the fd */
void tstream_write(thread t, void *buffer, int bufsz);
void tstream_write_buffer(thread t, tstream_buffer buffer);

/* thread internal utils */
void *_tstream_main(void *arg);
thread _tstream_new(int fd, int port, char *host, tstream_handler_read read, tstream_handler_err err, void *data);
int _tstream_connect(int port, char *host);
void _tstream_write(thread t);
void _tstream_cleanup(void *arg);
void _tstream_cleanup_evt(void *arg);
int _tstream_socket(int port, struct in_addr *iap);

#endif
