/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <jabber/jlib.h>
#include <config.h>

int jlib_log_device = JLIB_LOG_NONE;
thread jlib_log_device_file = NULL;

void _jdebug(char *zone, const char *msgfmt, ...)
{
	va_list ap;
	static char loghdr[LOGSIZE_HDR];
	static char logmsg[LOGSIZE_TAIL];
	static int size;

	/* XXX: We may want to check the sizes eventually */
	size = snprintf(loghdr, LOGSIZE_HDR, "%s debug/%s %s\n", 
					create_log_timestamp(),zone, msgfmt);

	va_start(ap, msgfmt);
	size = vsnprintf(logmsg, LOGSIZE_TAIL, loghdr, ap);
    
	_log(logmsg);

	return;
}

char *zonestr(char *file, int line)
{
    static char buff[64];
    char *dot, *f;

    f = strdup(file);
    dot = strstr(f,".");
    if(dot != NULL)
        *dot = '\0';

    snprintf(buff,63,"%s:%d",f,line);

    free(f);
    return buff;
}

char *create_log_timestamp(void)
{
	time_t t;
	int ret, sz;
	char *tmp_str;

	t = time(NULL);

	if(t == (time_t)-1)
		return NULL; 
	
	tmp_str = ctime(&t);
	sz = strlen(tmp_str);
	/* chop off the \n */
	tmp_str[sz-1]=' ';
	if(ret == -1)
		return NULL;

	return tmp_str;
}

void jwarn(char *zone, const char *msgfmt, ...)
{
	va_list ap;
	static char loghdr[LOGSIZE_HDR];
	static char logmsg[LOGSIZE_HDR];
	static int size;

	/* XXX: We may want to check the sizes eventually */
	size = snprintf(loghdr, LOGSIZE_HDR, "%s warn/%s %s\n",
					create_log_timestamp(), zone, msgfmt);

	va_start(ap, msgfmt);
	size = vsnprintf(logmsg, LOGSIZE_TAIL, loghdr, ap);
    
	_log(logmsg);

	return;
}

void jerror(char *zone, const char *msgfmt, ...)
{
	va_list ap;
	static char loghdr[LOGSIZE_HDR];
	static char logmsg[LOGSIZE_HDR];
	static int size;

	/* XXX: We may want to check the sizes eventually */
	size = snprintf(loghdr, LOGSIZE_HDR, "%s error/%s %s\n", 
					create_log_timestamp(), zone, msgfmt);

	va_start(ap, msgfmt);
	size = vsnprintf(logmsg, LOGSIZE_TAIL, loghdr, ap);
    
	_log(logmsg);

	return;
}

void _log(const char *logmsg)
{
    switch(jlib_log_device)
    {
        case JLIB_LOG_NONE:
            break;
        case JLIB_LOG_STDERR:
            fprintf(stderr,"%s", logmsg);
            break;
        case JLIB_LOG_SYSLOG:
            syslog(LOG_NOTICE, logmsg);
            break;
        case JLIB_LOG_FILE:
            tstream_write(jlib_log_device_file, (char *)logmsg, -1);
            break;
    }
}

